﻿using System.Data;
using System.Diagnostics;
using MySql.Data.MySqlClient;

namespace SQL_Tool
{
    class MySqlWrapper : DatabaseWrapper
    {
        private MySqlConnection _dbcon = null;
        private long _execTime;

        public override ConnectionState State => _dbcon.State;

        public MySqlWrapper(string connectionString)
        {
            _dbcon = new MySqlConnection(connectionString);
        }

        public MySqlWrapper(string host, string database, string username, string password)
        {
            MySqlConnectionStringBuilder csb = new MySqlConnectionStringBuilder
            {
                Server = host,
                Database = database,
                UserID = username,
                Password = password
            };
            _dbcon = new MySqlConnection(csb.ToString());
            _dbcon.Open();
            System.Console.WriteLine("MySQL version : {0}", _dbcon.ServerVersion);
        }

        ~MySqlWrapper()
        {
            if( _dbcon != null && _dbcon.State == ConnectionState.Open)
            {
                _dbcon.Close();
            }
        }

        public override int ExecuteCommand(string sqlStatement)
        {
            Stopwatch sw = new Stopwatch();
            sw.Start();
            MySqlCommand cmd = new MySqlCommand(sqlStatement, _dbcon);
            var rv = cmd.ExecuteNonQuery();
            sw.Stop();
            _execTime = sw.ElapsedMilliseconds;
            return rv;
        }

        public override DataTable ExecuteQuery(string sqlStatement, DataTable dt = null)
        {
            if (dt == null)
            {
                dt = new DataTable();
            }
            Stopwatch sw = new Stopwatch();
            sw.Start();
            try
            {
                MySqlDataAdapter da = new MySqlDataAdapter(sqlStatement, _dbcon);
                da.Fill(dt);
            }
            catch (MySqlException ex)
            {
                throw new DatabaseWrapperInvalidQueryException(ex.Message);
            }
            sw.Stop();
            _execTime = sw.ElapsedMilliseconds;
            return dt;
        }

        public override long GetExecutionTime()
        {
            return _execTime;
        }

    }
}
